//_____________________________________________________________
//  Simulation of stand structure change   (c)  Takashi Kohyama
//   since 90/02/05 ~ for Sumatra Plots  
//
//  ver. 04/04 CONC.BAS
//  ver. 90/12/01   for 14 spp  S-MODEL   Yakushima
//_____________________________________________________________
//  <<GAP MODEL>>: incorporating patch mosaic   29/Jul/1991 ~
//  GapM.bas for multi-species system           16/Aug/1991 ~ 
//  Publication: Kohyama T (1993) Journal of Ecology 81:131-143
//_____________________________________________________________
//  Version for SERIMBU DATA analysis           25/Jan/1999 ~
//  Translated to C++                           19/Mar/1999 ~
//  Completed by Matthew Potts                  22/Mar/1999 ~
//
//  Patch Demography test version; setting no_gap_formation patch age threshold
//  16/Nov/2005 onward; for Simon Levin Special Issue of Ecol. Res.
//
//  This model is used for the following publication:
//  Kohyama, T. (2006) "The effect of patch demography on the community structure
//      of forest trees"  Ecological Research 21, in press.
//  (c) Takashi Kohyama, all right reserved. 
 

#include <iostream> 
#include <fstream> 
#include <cstdlib> 
#include <cmath>

using namespace std;

#define dpatchage 101
#define dsize_class 201 
#define nspnn 101

class patch_tree {
        public:
        double tree_size[dpatchage][dsize_class];
};

class tree {
        public:
        double size[dsize_class];
};

class patch {	
        public:
        double patch_age[dpatchage];
};

class Species {
        public:
        char name[10];     // # characters for each species name
};

class Data {
        public:
        double time;
        double area;
        double density;
        double BA;
        // double biomass;
};

int main() {

// variable; parameter setting

int i, j, k, t,spn,numout;
int init, maxt, maxx, maxa, time, dur;
double dx, maximt, maximx, minimx, maxima, mx, x;
double dt, da, dadt;
double aver_duration, no_mort_yr, gap_form_rate;
const double PI_quart = 3.1416/4;

 char datainfile[100], dataoutfile[100], dataoutfile2[100], dataoutfile3[100];

spn = 12;                                 //number of species

aver_duration = 100;  // reverse of average gap formation rate, in years; 
//cout << "input no-mortality duration in year: " ;
//cin >> no_mort_yr;		// CHANGE HERE for different patch demography

no_mort_yr = 0;		// 'lag phase' of Kohyama (2006)
gap_form_rate = 1/(aver_duration - no_mort_yr);

dt = 1;                                  // time interval
dx = 2;                                  // size interval
da = 10;                                 // stand age interval
init = 0;                                // initial time
maximt = 4000;                  // maximum time
maximx = 150;                      // maximum size
minimx = 0;                           // minimum size
maxima = 400;                              // maximum stand age/da
mx = 2;                                  // for mean x of class
numout = 500;

maxx=ceil(maximx/dx-0.001);
maxa=ceil(maxima/da-0.001);
maxt=ceil(maximt/dt-0.001);
dadt = da/dt;

Species plname[nspnn];

// dynamic equations variables, with initialization

// Data *Output;
// Output = (Data *) calloc(numout,sizeof(Data));

patch_tree *F;
F = (patch_tree *) calloc(nspnn, sizeof(patch_tree));

tree *F0, *Fa1;
F0 = (tree *) calloc(nspnn,sizeof(tree));
Fa1 = (tree *) calloc(nspnn,sizeof(tree));

patch *B0;
B0 = (patch *) calloc(nspnn,sizeof(patch));

double *B, *Btot,*R0, *Blocal,*dens;
double *S, *S1;
B = (double *) calloc(dsize_class,sizeof(double));
Btot = (double *) calloc(nspnn,sizeof(double));
R0 = (double *) calloc(nspnn,sizeof(double));	// for OS_recr
Blocal = (double *) calloc(dpatchage,sizeof(double));
dens = (double *) calloc(nspnn,sizeof(double));
S = (double *) calloc(dpatchage,sizeof(double));
S1 = (double *) calloc(dpatchage,sizeof(double));


double Sa = 0, Sa1 = 0, Srecr = 0;                // .. require a fine precision

// dynamics temporary variables (derivative abbreviation from above)

double freq, Fa, G, GFx, GFx1, dFdt, dFda, dGFdx, mc, surv, recr;  //, R0;
double ms, dSdt;
double barea;
// demographic parameters definition

double a[nspnn], a1[nspnn], a2[nspnn];            // growth parameters
double c[nspnn], c1[nspnn],                    // mortality parameters
      d[nspnn], d1[nspnn];                    // recruitment parameters

//  Input demographic parameters from external file

strcpy(datainfile, "twelve_paramCoex.dat");       
ifstream infile("twelve_paramCoex.dat", ios::in);


while (!infile.eof()) {
        for (j=1; j<=spn; j++) {
	infile >> plname[j].name;
	infile >> a[j] >> a1[j] >> a2[j];
	infile >> c[j] >> c1[j];
	infile >> d[j] >> d1[j];
        };
};
infile.close();

double rs;

char ofilname[20];
//cout << "Name of output file:  ";
//cin >> ofilname;
strcpy(dataoutfile, ofilname);  //"Gmodel.out");
strcpy(dataoutfile,"BATime.out");
strcpy(dataoutfile2,"AgeDist.out");
strcpy(dataoutfile3,"DBHDist.out");
ofstream outfile(dataoutfile);
ofstream outfile2(dataoutfile2);
ofstream outfile3(dataoutfile3);
//      cout << AA << "\t" << HH << "\t" << a[j] << "\t" << a1[j] << "\t" << a2[j]<< "\t"  << c[j] << "\t" << c1[j] << "\t" << d[j] << "\t" << d1[j] << endl;

// Output simulation conditions prior to dynamics_loop

outfile << "running Gmodel.c:  Multispecies, gap mosaic " << endl;
outfile << "dt " << dt <<" yr" << endl;
outfile << "da " << da <<" yr" << endl;
outfile << "dx " << dx <<" cm" << endl;
outfile << " minus G not allowed" << endl;
outfile << endl;
outfile << "no patch-death duration (yr)     " << no_mort_yr  << endl;
outfile << "patch mortality afterword (1/yr) " << gap_form_rate << endl;
outfile << endl;
outfile << " species parameters (b, b1, b2, c, c1, d, d1): " << endl;

for (j=1; j<=spn; j++) {
        outfile << a[j] << "\t" << a1[j] << "\t" << a2[j] << "\t"  
	<< c[j] << "\t" << c1[j] << "\t" << d[j] << "\t" << d1[j] << endl;
		cout << d[j] << "\t";
};

outfile << "   Start of run               " << endl;
outfile << endl;
cout << endl;
//------------------------------------------------ changing demography parameters 
for (dur=0; dur<=0; dur++)
{
//no_mort_yr = dur*10;
//gap_form_rate = 1/(aver_duration - no_mort_yr);
//outfile << endl;
cout << endl;
outfile << " -----------------------------------------------------" << endl;
outfile << "no mort year: " << no_mort_yr << endl;
cout << "no mort year: " << no_mort_yr << endl;

//
// Fj(t,a,x) = s(t,a)*pj(t,a,x)   pj(t,a,x) is local density per unit area
//
// Initial condition: all at minimum patch age, 0 (or 1) trees per m^2


for (k=0; k<=maxa; k++)
{
    if (k < 100/da)
		S[k] = da/100;       // uniform distribution up to patch age = 100 yr
    else 
		S[k] = 0;
	
    for (i=1; i<=maxx; i++)
    {
		x = (i - 1)*dx + minimx + mx;
		for (j=1; j<=spn; j++)
		{
	        if (i <= 1 & k < 100/da) 
				F[j].tree_size[k][i] = 0.001;	//  1 is one tree per m2
	        else
				F[j].tree_size[k][i] = 0;
		    
	        B0[j].patch_age[k] += x*x*F[j].tree_size[k][i]*PI_quart;
		};
    };
};

double SS = 0;          // to check whether area conserved at 1
double den = 0;               // for cumulation of tree number; per sq-m
double Bt = 0;                // for basal area cumulation
for (j=1; j<=spn; j++) {
        Btot[j] = 0;
		R0[j] = 0;	// for OS_recr
};
int count = 1;

//  Dynamics main t loop
//
//_______________________________________________________________________

outfile << "Time" <<"\t" <<"Area" << "\t" << "Density" <<"\t"  << "Total_BA" << endl;

for (t=init; t<=maxt; t++)
{
	time = (t*dt)/20;
	
	if (t*dt == time*20)      // return change at t*100*dt yr interval
	{	
		for (j=1; j<=spn; j++) 
		{
			Btot[j] = 0.;
			R0[j] = 0.;
		};
		for (k=0; k<=maxa; k++)
		{
	        SS += S[k];
	        for (j=1; j<=spn; j++)
	        {
				for (i=1; i<=maxx; i++)
				{
					x = (i - 1)*dx + minimx + mx;
					den += F[j].tree_size[k][i];
					Bt += x*x*F[j].tree_size[k][i];
					Btot[j] += x*x*F[j].tree_size[k][i]*PI_quart;
          	        
				};
	        };
		};
 		
		Bt = Bt*PI_quart;
		
		count ++;
		
		outfile << t*dt << "\t" << SS << "\t" << den << "\t" << Bt << "\t" ;
		cout  << t*dt << "\t" ;
		
		for (j=spn; j>=1; j--)
		{
	        outfile << Btot[j]<< "\t" ;
	        cout << Btot[j] << "\t" ;
		};
		outfile << endl;
		cout << endl;
    
		SS = 0.;
		Bt = 0.; 
		den = 0.; 
 
	};
  

	for (j=1; j<=spn; j++)
	{
		Btot[j] = 0;
		R0[j] = 0;
	};
  	Sa1 = Srecr = 0;
  
	for (k=0; k<=maxa; k++)
	{
		for (j=1; j<=spn; j++)
		{
			B0[j].patch_age[k] = 0;
		}
    
	//   Cumulative basal area routine
   
		for (i=maxx; i>=1; i--)
		{
			B[i] = 0;
      			x = (i - 1)*dx + minimx + mx;
			for (j=1; j<=spn; j++)
			{
				freq = F[j].tree_size[k][i];
				
				if (S[k]==0)
					B0[j].patch_age[k]=0;
				else
				  B0[j].patch_age[k] += x*x*freq*PI_quart/S[k];
				        
        
				B[i] += B0[j].patch_age[k];               //  cumulative basal area at each k
			};
		};
    
		Blocal[k] = B[1];
    
		for (j=1; j<=spn; j++)
		{
			Btot[j] += B0[j].patch_age[k]*S[k];         // species basal area
		}
   
	//    Stand age distribution routine
  
		Sa = S[k]/da;
		if (k==maxa)
			Sa = 0;
		// PATCH DEMOGRAPHY DEFINITION by 'ms' as function of stand age k
		if (k < no_mort_yr/da)
			ms = 0;
		else
			ms = gap_form_rate;
		
		Srecr += ms*S[k];
    
	//     Tree size distribution routine
   
		for (j=1; j<=spn; j++)
		{
			for (i=1; i<=maxx; i++)
			{
				if (i==1)
        				GFx1 = 0;
				freq=F[j].tree_size[k][i];
                                               //  (1) size x dynamics
				x = (i - 1)*dx + minimx + mx;
				G = a[j]*x*(1 - a1[j]*log(x) - a2[j]*B[i]);
				if (G<0)                                    // negative G not allowed
					G = 0;
				GFx = G*freq/dx;
				dGFdx = GFx - GFx1;

												//  (1.5) reproduction at size x; for OS_recr
				recr = d[j]*freq*x*x*PI_quart*(1 - d1[j]*B[i]);
				if (recr<0)
					recr = 0;
				R0[j] += recr;
				
                                               //   (2) age a dynamics
				Fa = freq/da;
        			if (k==maxa)
        				Fa = 0;
        			if (k==0)
        				Fa1[j].size[i] = 0;
				dFda = Fa - Fa1[j].size[i];
                                               //   (3) mortality by competition
				// fruction of survival with gap formation
         
				surv = dur*ms/(dur+x); // to allow advance regeneration, in Kohyama (2006)
					    // where 'dur' means 'k' of Eq (10), Kohyama (2006)
				mc = surv + c1[j]*B[i]; // Density-dependent mortality for 'mc'; 
										// same as Thin-M model (Kohyama 1992)

                                               //   (4) gap formation
				F0[j].size[i] += surv*freq;   // summation of adv. regen. with respect to k
                                               //   (5) main dynamics equation
				dFdt = -dFda - dGFdx - ms*freq - mc*freq;
				F[j].tree_size[k][i] = freq + dFdt*dt;
        
				Fa1[j].size[i] = Fa;
				GFx1 = GFx;
        
			};
		};
    
    
	//  Main equation for dynamics of patch age distribution
    
		dSdt = -Sa + Sa1 - ms*S[k];
		S1[k] = S[k];
		S[k] = S1[k] + dSdt*dt;
	
		Sa1 = Sa;
    
	};
  
//     Boundary conditions
  
//     (1) recruitment at x=minx in each stand age class, from seedling bank
  
	for (k=0; k<=maxa; k++)
	{
  	  for (j=1; j<=spn; j++)
		{
		//	R0 = d[j]*Btot[j]*exp(-d1[j]*Blocal[k])*dt; 
		//	if (R0<0) 
		//		R0=0;
			F[j].tree_size[k][1] += R0[j]*S1[k]*dt;
		};
	};
  
//    (2) advance regeneration at a=0, supplied by gap formation
  
	S[0] = S[0] + Srecr*dt;
  
	for (j=1; j<=spn; j++)
	{    
		for (i=1; i<=maxx; i++)
		{
			F[j].tree_size[0][i] += F0[j].size[i]*dt;
			F0[j].size[i] = 0;                         //  resetting of adv. regen. for next t
		};
	};
  
};  

// End of dynamics t loop
//_______________________________________________________________________
// Final state 

outfile2 << "time : " << t*dt << endl;

SS = 0;

// patch frequency distribution and basal area per sq-m in each patch age

outfile2 << "Age" << "\t" << "Freq" << "\t" << "Cum_freq";
for(j=spn; j>=1; j--) outfile2 << "\t" << "Local_BA";
outfile2 << endl;

for (k=0; k<=maxa; k++)
{
	SS = SS + S[k];
	outfile2 << (k+.5)*da << "\t" << S[k] << "\t" << SS;
	for (j=spn; j>=1; j--)
		{
			outfile2  << "\t" << B0[j].patch_age[k];
		}
	outfile2 << endl;
}

// patch-specific and total size distribution, converted as per 1-ha
double P;
outfile3 << "dbh distribution" << endl;

// for (k=0; k<=maxa; k++)
// {
//	outfile3 << (k+.5)*da << "\t";
// } 
// outfile3 << "all" << endl;

for (i=1; i<=maxx; i++)
{
	outfile3 << (i - 1)*dx + minimx + mx << "\t";
	x = (i - 1)*dx + minimx + mx;
	freq = 0;
	for (j=spn; j>=1; j--)
	{
	barea = 0.0;
	    for (k=0; k<=maxa; k++)
	    {
		freq = freq + F[j].tree_size[k][i]*S[k];
		barea = barea + x*x*F[j].tree_size[k][i]*PI_quart;
	    }
	    //	    outfile3 << freq*10000 <<  "\t";
	    outfile3 << barea <<  "\t";
	    freq = 0.;
	    barea = 0.0;
	}
	outfile3 << endl;
}
cout << endl;
}

cout << "____END OF RUN____" << endl;

return 0;
}
